// This sample code tests the performance of sorting provided by
// "szsort.h". Random 7-letters strings are sorted by
// StringSort::Sort() and std::sort().
#include "szsort.h"

#include <cassert>
#include <ctime>

#include <vector>

using namespace std;


namespace {

using std::size_t;

// Buffer size (how many keys are sorted in this test).
const size_t N = 1 << 20;

// Structure for random 7-letters keys.
typedef struct Key {
  char s[8];
} Key;

// Buffer to be sorted.
Key Buf[N];

// Timer.
void Timer(const char *message) {
  static clock_t cl;
  if (message == NULL)
    cl = clock();
  else
    printf("%s%.3f sec\n", message, 1.0 * (clock() - cl) / CLOCKS_PER_SEC);
}

// Randomize Keys in buffer.
template <typename RandomAccessIterator>
void RandomizeBuffer(RandomAccessIterator l, RandomAccessIterator r) {
  while (l < r) {
    for (size_t i = 0; i < sizeof(l->s) - 1; ++i)
      l->s[i] = '0' + rand() % 10;
    l->s[sizeof(l->s) - 1] = '\0';
    ++l;
  }
}

// Confirm that an array is sorted.
template <typename RandomAccessIterator>
void IsSorted(RandomAccessIterator l, RandomAccessIterator r) {
  while (++l < r)
    assert(strcmp((l - 1)->s, l->s) <= 0);
}

// Function object to handle same keys.
struct HandleSameKeys {
  template <typename RandomAccessIterator>
  bool operator()(RandomAccessIterator l, RandomAccessIterator r) {
    printf("HandleSameKeys: stop sorting because of same keys\n");
    return false;
  }
};

// Function object to get a letter from a key.
struct GetLetter {
  unsigned char operator()(const Key &key, size_t depth) {
    return (unsigned char)key.s[depth];
  }
};

// Function object to compare a pair of keys (for std::sort()).
struct LessThan {
  bool operator()(const Key &lhs, const Key &rhs) {
    return strcmp(lhs.s, rhs.s) < 0;
  }
};

}  // namespace.

int main() {
  srand((unsigned)time(NULL));

  // Sorting for an array.
  RandomizeBuffer(Buf, Buf + N);
  Timer(NULL);
  StringSort::Sort(Buf, Buf + N, 0, GetLetter());
  Timer("StringSort::Sort() for array: ");
  IsSorted(Buf, Buf + N);

  // Sorting will stop if same keys are in the buffer.
  RandomizeBuffer(Buf, Buf + N);
  Timer(NULL);
  bool sorted = StringSort::Sort(Buf, Buf + N, 0,
                                 GetLetter(), HandleSameKeys());
  Timer("StringSort::Sort() for array with HandleSameKeys(): ");
  if (sorted)
    IsSorted(Buf, Buf + N);

  // Sorting for a vector.
  vector<Key> vec(N);
  RandomizeBuffer(vec.begin(), vec.end());
  Timer(NULL);
  StringSort::Sort(vec.begin(), vec.end(), 0, GetLetter());
  Timer("StringSort::Sort() for vector: ");
  IsSorted(vec.begin(), vec.end());

  // Sorting will stop if same keys are in the buffer.
  RandomizeBuffer(vec.begin(), vec.end());
  Timer(NULL);
  sorted = StringSort::Sort(vec.begin(), vec.end(), 0,
                            GetLetter(), HandleSameKeys());
  Timer("StringSort::Sort() for vector with HandleSameKeys(): ");
  if (sorted)
    IsSorted(Buf, Buf + N);

  // Standard sorting algorithm.
  RandomizeBuffer(vec.begin(), vec.end());
  Timer(NULL);
  sort(vec.begin(), vec.end(), LessThan());
  Timer("std::sort() for vector: ");
  IsSorted(vec.begin(), vec.end());

  return 0;
}
